@@ -7,7 +7,6 @@
 #include "d3d11_device.h"
 #include "d3d11_enums.h"
 #include "d3d11_interop.h"
-#include "d3d11_present.h"
 
 namespace dxvk {
   Logger Logger::s_instance("d3d11.log");
@@ -23,15 +22,17 @@ extern "C" {
     const D3D_FEATURE_LEVEL*  pFeatureLevels,
           UINT                FeatureLevels,
           ID3D11Device**      ppDevice) {
+    InitReturnPtr(ppDevice);
+
     Rc<DxvkAdapter>  dxvkAdapter;
     Rc<DxvkInstance> dxvkInstance;
 
-    Com<IDXGIVkAdapter> dxgiVkAdapter;
+    Com<IDXGIDXVKAdapter> dxgiVkAdapter;
     
     // Try to find the corresponding Vulkan device for the DXGI adapter
-    if (SUCCEEDED(pAdapter->QueryInterface(__uuidof(IDXGIVkAdapter), reinterpret_cast<void**>(&dxgiVkAdapter)))) {
+    if (SUCCEEDED(pAdapter->QueryInterface(__uuidof(IDXGIDXVKAdapter), reinterpret_cast<void**>(&dxgiVkAdapter)))) {
       dxvkAdapter  = dxgiVkAdapter->GetDXVKAdapter();
-      dxvkInstance = dxvkAdapter->instance();
+      dxvkInstance = dxgiVkAdapter->GetDXVKInstance();
     } else {
       Logger::warn("D3D11CoreCreateDevice: Adapter is not a DXVK adapter");
       DXGI_ADAPTER_DESC desc;
@@ -70,7 +71,7 @@ extern "C" {
     for (flId = 0 ; flId < FeatureLevels; flId++) {
       Logger::info(str::format("D3D11CoreCreateDevice: Probing ", pFeatureLevels[flId]));
       
-      if (D3D11Device::CheckFeatureLevelSupport(dxvkAdapter, pFeatureLevels[flId]))
+      if (D3D11Device::CheckFeatureLevelSupport(dxvkInstance, dxvkAdapter, pFeatureLevels[flId]))
         break;
     }
     
@@ -85,7 +86,7 @@ extern "C" {
     try {
       Logger::info(str::format("D3D11CoreCreateDevice: Using feature level ", fl));
       Com<D3D11DXGIDevice> device = new D3D11DXGIDevice(
-        pAdapter, dxvkAdapter.ptr(), fl, Flags);
+        pAdapter, dxvkInstance, dxvkAdapter, fl, Flags);
       
       return device->QueryInterface(
         __uuidof(ID3D11Device),
@@ -97,6 +98,111 @@ extern "C" {
   }
   
   
+  static HRESULT D3D11InternalCreateDeviceAndSwapChain(
+          IDXGIAdapter*         pAdapter,
+          D3D_DRIVER_TYPE       DriverType,
+          HMODULE               Software,
+          UINT                  Flags,
+    const D3D_FEATURE_LEVEL*    pFeatureLevels,
+          UINT                  FeatureLevels,
+          UINT                  SDKVersion,
+    const DXGI_SWAP_CHAIN_DESC* pSwapChainDesc,
+          IDXGISwapChain**      ppSwapChain,
+          ID3D11Device**        ppDevice,
+          D3D_FEATURE_LEVEL*    pFeatureLevel,
+          ID3D11DeviceContext** ppImmediateContext) {
+    InitReturnPtr(ppDevice);
+    InitReturnPtr(ppSwapChain);
+    InitReturnPtr(ppImmediateContext);
+
+    if (pFeatureLevel)
+      *pFeatureLevel = D3D_FEATURE_LEVEL(0);
+
+    HRESULT hr;
+
+    Com<IDXGIFactory> dxgiFactory = nullptr;
+    Com<IDXGIAdapter> dxgiAdapter = pAdapter;
+    Com<ID3D11Device> device      = nullptr;
+    
+    if (ppSwapChain && !pSwapChainDesc)
+      return E_INVALIDARG;
+    
+    if (!pAdapter) {
+      // We'll treat everything as hardware, even if the
+      // Vulkan device is actually a software device.
+      if (DriverType != D3D_DRIVER_TYPE_HARDWARE)
+        Logger::warn("D3D11CreateDevice: Unsupported driver type");
+      
+      // We'll use the first adapter returned by a DXGI factory
+      hr = CreateDXGIFactory1(__uuidof(IDXGIFactory), reinterpret_cast<void**>(&dxgiFactory));
+
+      if (FAILED(hr)) {
+        Logger::err("D3D11CreateDevice: Failed to create a DXGI factory");
+        return hr;
+      }
+
+      hr = dxgiFactory->EnumAdapters(0, &dxgiAdapter);
+
+      if (FAILED(hr)) {
+        Logger::err("D3D11CreateDevice: No default adapter available");
+        return hr;
+      }
+    } else {
+      // We should be able to query the DXGI factory from the adapter
+      if (FAILED(dxgiAdapter->GetParent(__uuidof(IDXGIFactory), reinterpret_cast<void**>(&dxgiFactory)))) {
+        Logger::err("D3D11CreateDevice: Failed to query DXGI factory from DXGI adapter");
+        return E_INVALIDARG;
+      }
+      
+      // In theory we could ignore these, but the Microsoft docs explicitly
+      // state that we need to return E_INVALIDARG in case the arguments are
+      // invalid. Both the driver type and software parameter can only be
+      // set if the adapter itself is unspecified.
+      // See: https://msdn.microsoft.com/en-us/library/windows/desktop/ff476082(v=vs.85).aspx
+      if (DriverType != D3D_DRIVER_TYPE_UNKNOWN || Software)
+        return E_INVALIDARG;
+    }
+    
+    // Create the actual device
+    hr = D3D11CoreCreateDevice(
+      dxgiFactory.ptr(), dxgiAdapter.ptr(),
+      Flags, pFeatureLevels, FeatureLevels,
+      &device);
+    
+    if (FAILED(hr))
+      return hr;
+    
+    // Create the swap chain, if requested
+    if (ppSwapChain) {
+      DXGI_SWAP_CHAIN_DESC desc = *pSwapChainDesc;
+      hr = dxgiFactory->CreateSwapChain(device.ptr(), &desc, ppSwapChain);
+
+      if (FAILED(hr)) {
+        Logger::err("D3D11CreateDevice: Failed to create swap chain");
+        return hr;
+      }
+    }
+    
+    // Write back whatever info the application requested
+    if (pFeatureLevel)
+      *pFeatureLevel = device->GetFeatureLevel();
+    
+    if (ppDevice)
+      *ppDevice = device.ref();
+    
+    if (ppImmediateContext)
+      device->GetImmediateContext(ppImmediateContext);
+
+    // If we were unable to write back the device and the
+    // swap chain, the application has no way of working
+    // with the device so we should report S_FALSE here.
+    if (!ppDevice && !ppImmediateContext && !ppSwapChain)
+      return S_FALSE;
+    
+    return S_OK;
+  }
+  
+
   DLLEXPORT HRESULT __stdcall D3D11CreateDevice(
           IDXGIAdapter*         pAdapter,
           D3D_DRIVER_TYPE       DriverType,
@@ -108,70 +214,11 @@ extern "C" {
           ID3D11Device**        ppDevice,
           D3D_FEATURE_LEVEL*    pFeatureLevel,
           ID3D11DeviceContext** ppImmediateContext) {
-    Com<IDXGIFactory> dxgiFactory = nullptr;
-    Com<IDXGIAdapter> dxgiAdapter = pAdapter;
-    
-    if (dxgiAdapter == nullptr) {
-      // We'll treat everything as hardware, even if the
-      // Vulkan device is actually a software device.
-      if (DriverType != D3D_DRIVER_TYPE_HARDWARE)
-        Logger::warn("D3D11CreateDevice: Unsupported driver type");
-      
-      // We'll use the first adapter returned by a DXGI factory
-      if (FAILED(CreateDXGIFactory1(__uuidof(IDXGIFactory), reinterpret_cast<void**>(&dxgiFactory)))) {
-        Logger::err("D3D11CreateDevice: Failed to create a DXGI factory");
-        return E_FAIL;
-      }
-      
-      if (FAILED(dxgiFactory->EnumAdapters(0, &dxgiAdapter))) {
-        Logger::err("D3D11CreateDevice: No default adapter available");
-        return E_FAIL;
-      }
-      
-    } else {
-      // We should be able to query the DXGI factory from the adapter
-      if (FAILED(dxgiAdapter->GetParent(__uuidof(IDXGIFactory), reinterpret_cast<void**>(&dxgiFactory)))) {
-        Logger::err("D3D11CreateDevice: Failed to query DXGI factory from DXGI adapter");
-        return E_FAIL;
-      }
-      
-      // In theory we could ignore these, but the Microsoft docs explicitly
-      // state that we need to return E_INVALIDARG in case the arguments are
-      // invalid. Both the driver type and software parameter can only be
-      // set if the adapter itself is unspecified.
-      // See: https://msdn.microsoft.com/en-us/library/windows/desktop/ff476082(v=vs.85).aspx
-      if (DriverType != D3D_DRIVER_TYPE_UNKNOWN || Software != nullptr)
-        return E_INVALIDARG;
-    }
-    
-    // Create the actual device
-    Com<ID3D11Device> device;
-    
-    HRESULT hr = D3D11CoreCreateDevice(
-      dxgiFactory.ptr(), dxgiAdapter.ptr(),
-      Flags, pFeatureLevels, FeatureLevels,
-      &device);
-    
-    if (FAILED(hr))
-      return hr;
-    
-    // Write back whatever info the application requested
-    if (pFeatureLevel != nullptr)
-      *pFeatureLevel = device->GetFeatureLevel();
-    
-    if (ppDevice != nullptr)
-      *ppDevice = device.ref();
-    
-    if (ppImmediateContext != nullptr)
-      device->GetImmediateContext(ppImmediateContext);
-    
-    // If we were unable to write back the device and the
-    // swap chain, the application has no way of working
-    // with the device so we should report S_FALSE here.
-    if (ppDevice == nullptr && ppImmediateContext == nullptr)
-      return S_FALSE;
-    
-    return S_OK;
+    return D3D11InternalCreateDeviceAndSwapChain(
+      pAdapter, DriverType, Software, Flags,
+      pFeatureLevels, FeatureLevels, SDKVersion,
+      nullptr, nullptr,
+      ppDevice, pFeatureLevel, ppImmediateContext);
   }
   
   
@@ -188,56 +235,11 @@ extern "C" {
           ID3D11Device**        ppDevice,
           D3D_FEATURE_LEVEL*    pFeatureLevel,
           ID3D11DeviceContext** ppImmediateContext) {
-    Com<ID3D11Device>        d3d11Device;
-    Com<ID3D11DeviceContext> d3d11Context;
-    
-    if (ppSwapChain && !pSwapChainDesc)
-      return E_INVALIDARG;
-    
-    // Try to create a device first.
-    HRESULT status = D3D11CreateDevice(pAdapter, DriverType,
-      Software, Flags, pFeatureLevels, FeatureLevels,
-      SDKVersion, &d3d11Device, pFeatureLevel, &d3d11Context);
-    
-    if (FAILED(status))
-      return status;
-    
-    // Again, the documentation does not exactly tell us what we
-    // need to do in case one of the arguments is a null pointer.
-    if (ppSwapChain) {
-      Com<IDXGIDevice>  dxgiDevice  = nullptr;
-      Com<IDXGIAdapter> dxgiAdapter = nullptr;
-      Com<IDXGIFactory> dxgiFactory = nullptr;
-      
-      if (FAILED(d3d11Device->QueryInterface(__uuidof(IDXGIDevice), reinterpret_cast<void**>(&dxgiDevice)))) {
-        Logger::err("D3D11CreateDeviceAndSwapChain: Failed to query DXGI device");
-        return E_FAIL;
-      }
-      
-      if (FAILED(dxgiDevice->GetParent(__uuidof(IDXGIAdapter), reinterpret_cast<void**>(&dxgiAdapter)))) {
-        Logger::err("D3D11CreateDeviceAndSwapChain: Failed to query DXGI adapter");
-        return E_FAIL;
-      }
-      
-      if (FAILED(dxgiAdapter->GetParent(__uuidof(IDXGIFactory), reinterpret_cast<void**>(&dxgiFactory)))) {
-        Logger::err("D3D11CreateDeviceAndSwapChain: Failed to query DXGI factory");
-        return E_FAIL;
-      }
-      
-      DXGI_SWAP_CHAIN_DESC desc = *pSwapChainDesc;
-      if (FAILED(dxgiFactory->CreateSwapChain(d3d11Device.ptr(), &desc, ppSwapChain))) {
-        Logger::err("D3D11CreateDeviceAndSwapChain: Failed to create swap chain");
-        return E_FAIL;
-      }
-    }
-    
-    if (ppDevice != nullptr)
-      *ppDevice = d3d11Device.ref();
-    
-    if (ppImmediateContext != nullptr)
-      *ppImmediateContext = d3d11Context.ref();
-    
-    return S_OK;
+    return D3D11InternalCreateDeviceAndSwapChain(
+      pAdapter, DriverType, Software, Flags,
+      pFeatureLevels, FeatureLevels, SDKVersion,
+      pSwapChainDesc, ppSwapChain,
+      ppDevice, pFeatureLevel, ppImmediateContext);
   }
   
 }
\ No newline at end of file
