@@ -112,7 +112,7 @@ namespace dxvk {
     if (!mapping.IsValid() && pDesc->Format != D3D9Format::NULL_FORMAT) {
       auto info = pDevice->UnsupportedFormatInfo(pDesc->Format);
 
-      if (pDesc->Pool != D3DPOOL_SCRATCH || info.elementSize == 0)
+      if (pDesc->Pool != D3DPOOL_SCRATCH || info->elementSize == 0)
         return D3DERR_INVALIDCALL;
     }
 
@@ -186,20 +186,20 @@ namespace dxvk {
   VkDeviceSize D3D9CommonTexture::GetMipSize(UINT Subresource) const {
     const UINT MipLevel = Subresource % m_desc.MipLevels;
 
-    const DxvkFormatInfo formatInfo = m_mapping.FormatColor != VK_FORMAT_UNDEFINED
-      ? *imageFormatInfo(m_mapping.FormatColor)
+    const DxvkFormatInfo* formatInfo = m_mapping.FormatColor != VK_FORMAT_UNDEFINED
+      ? imageFormatInfo(m_mapping.FormatColor)
       : m_device->UnsupportedFormatInfo(m_desc.Format);
 
     const VkExtent3D mipExtent = util::computeMipLevelExtent(
       GetExtent(), MipLevel);
     
     const VkExtent3D blockCount = util::computeBlockCount(
-      mipExtent, formatInfo.blockSize);
+      mipExtent, formatInfo->blockSize);
 
     const uint32_t planeCount = m_mapping.ConversionFormatInfo.PlaneCount;
 
     return std::min(planeCount, 2u)
-         * align(formatInfo.elementSize * blockCount.width, 4)
+         * align(formatInfo->elementSize * blockCount.width, 4)
          * blockCount.height
          * blockCount.depth;
   }
@@ -383,6 +383,7 @@ namespace dxvk {
 
   VkImageType D3D9CommonTexture::GetImageTypeFromResourceType(D3DRESOURCETYPE Type) {
     switch (Type) {
+      case D3DRTYPE_SURFACE:
       case D3DRTYPE_TEXTURE:       return VK_IMAGE_TYPE_2D;
       case D3DRTYPE_VOLUMETEXTURE: return VK_IMAGE_TYPE_3D;
       case D3DRTYPE_CUBETEXTURE:   return VK_IMAGE_TYPE_2D;
@@ -395,6 +396,7 @@ namespace dxvk {
           D3DRESOURCETYPE  Dimension,
           UINT             Layer) {
     switch (Dimension) {
+      case D3DRTYPE_SURFACE:
       case D3DRTYPE_TEXTURE:       return VK_IMAGE_VIEW_TYPE_2D;
       case D3DRTYPE_VOLUMETEXTURE: return VK_IMAGE_VIEW_TYPE_3D;
       case D3DRTYPE_CUBETEXTURE:   return Layer == AllLayers
@@ -446,7 +448,7 @@ namespace dxvk {
           UINT                   Layer,
           UINT                   Lod,
           VkImageUsageFlags      UsageFlags,
-          bool                   Srgb) {
+          bool                   Srgb) {    
     DxvkImageViewCreateInfo viewInfo;
     viewInfo.format    = m_mapping.ConversionFormatInfo.FormatColor != VK_FORMAT_UNDEFINED
                        ? PickSRGB(m_mapping.ConversionFormatInfo.FormatColor, m_mapping.ConversionFormatInfo.FormatSrgb, Srgb)
@@ -456,7 +458,7 @@ namespace dxvk {
     viewInfo.usage     = UsageFlags;
     viewInfo.type      = GetImageViewTypeFromResourceType(m_type, Layer);
     viewInfo.minLevel  = Lod;
-    viewInfo.numLevels  = m_desc.MipLevels - Lod;
+    viewInfo.numLevels = m_desc.MipLevels - Lod;
     viewInfo.minLayer  = Layer == AllLayers ? 0                : Layer;
     viewInfo.numLayers = Layer == AllLayers ? m_desc.ArraySize : 1;
 
