@@ -7,6 +7,12 @@
 
 namespace dxvk {
 
+  enum class D3D9FloatEmulation {
+    Disabled,
+    Enabled,
+    Strict
+  };
+
   struct D3D9Options {
 
     D3D9Options(const Rc<DxvkDevice>& device, const Config& config);
@@ -26,6 +32,9 @@ namespace dxvk {
     /// a higher value. May help with frame timing issues.
     int32_t maxFrameLatency;
 
+    /// Limit frame rate
+    int32_t maxFrameRate;
+
     /// Set the max shader model the device can support in the caps.
     int32_t shaderModel;
 
@@ -34,11 +43,6 @@ namespace dxvk {
 
     /// Whether or not to set the process as DPI aware in Windows when the API interface is created.
     bool dpiAware;
-    
-    /// Handle D3DLOCK_READONLY properly.
-    ///
-    /// Risen 1 writes to buffers mapped with readonly.
-    bool allowLockFlagReadonly;
 
     /// True:  Copy our constant set into UBO if we are relative indexing ever.
     /// False: Copy our constant set into UBO if we are relative indexing at the start of a defined constant
@@ -85,7 +89,7 @@ namespace dxvk {
     uint32_t maxAvailableMemory;
 
     /// D3D9 Floating Point Emulation (anything * 0 = 0)
-    bool d3d9FloatEmulation;
+    D3D9FloatEmulation d3d9FloatEmulation;
 
     /// Support the DF16 & DF24 texture format
     bool supportDFFormats;
@@ -96,11 +100,6 @@ namespace dxvk {
     /// Support D32
     bool supportD32;
 
-    /// SWVP Constant Limits
-    int32_t swvpFloatCount;
-    int32_t swvpIntCount;
-    int32_t swvpBoolCount;
-
     /// Disable D3DFMT_A8 for render targets.
     /// Specifically to work around a game
     /// bug in The Sims 2 that happens on native too!
@@ -157,10 +156,6 @@ namespace dxvk {
 
     /// Use device local memory for constant buffers.
     bool deviceLocalConstantBuffers;
-
-    /// Allow implicit discard of resources in contested situations.
-    /// Some naughty apps write to pointers outside of lock boundaries.
-    bool allowImplicitDiscard;
   };
 
 }
\ No newline at end of file
