@@ -1,5 +1,6 @@
 #include "d3d9_stateblock.h"
 
+#include "d3d9_caps.h"
 #include "d3d9_device.h"
 #include "d3d9_vertex_declaration.h"
 #include "d3d9_buffer.h"
@@ -38,6 +39,9 @@ namespace dxvk {
 
 
   HRESULT STDMETHODCALLTYPE D3D9StateBlock::Capture() {
+    if (m_captures.flags.test(D3D9CapturedStateFlag::VertexDecl))
+      SetVertexDeclaration(m_deviceState->vertexDecl.ptr());
+
     ApplyOrCapture<D3D9StateFunction::Capture>();
 
     return D3D_OK;
@@ -46,6 +50,10 @@ namespace dxvk {
 
   HRESULT STDMETHODCALLTYPE D3D9StateBlock::Apply() {
     m_applying = true;
+
+    if (m_captures.flags.test(D3D9CapturedStateFlag::VertexDecl) && m_state.vertexDecl != nullptr)
+      m_parent->SetVertexDeclaration(m_state.vertexDecl.ptr());
+
     ApplyOrCapture<D3D9StateFunction::Apply>();
     m_applying = false;
 
@@ -368,7 +376,7 @@ namespace dxvk {
   void D3D9StateBlock::CapturePixelSamplerStates() {
     m_captures.flags.set(D3D9CapturedStateFlag::SamplerStates);
 
-    for (uint32_t i = 0; i < 17; i++) {
+    for (uint32_t i = 0; i < caps::MaxTexturesPS + 1; i++) {
       m_captures.samplers.set(i, true);
 
       m_captures.samplerStates[i].set(D3DSAMP_ADDRESSU, true);
@@ -451,7 +459,7 @@ namespace dxvk {
   void D3D9StateBlock::CaptureVertexSamplerStates() {
     m_captures.flags.set(D3D9CapturedStateFlag::SamplerStates);
 
-    for (uint32_t i = 17; i < SamplerCount; i++) {
+    for (uint32_t i = caps::MaxTexturesPS + 1; i < SamplerCount; i++) {
       m_captures.samplers.set(i, true);
       m_captures.samplerStates[i].set(D3DSAMP_DMAPOFFSET, true);
     }
