@@ -78,10 +78,15 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE D3D9Texture2D::AddDirtyRect(CONST RECT* pDirtyRect) {
     if (pDirtyRect) {
       D3DBOX box = { UINT(pDirtyRect->left), UINT(pDirtyRect->top), UINT(pDirtyRect->right), UINT(pDirtyRect->bottom), 0, 1 };
-      m_texture.AddUpdateDirtyBox(&box, 0);
+      m_texture.AddDirtyBox(&box, 0);
     } else {
-      m_texture.AddUpdateDirtyBox(nullptr, 0);
+      m_texture.AddDirtyBox(nullptr, 0);
     }
+
+    // Some games keep using the pointer returned in LockRect() after calling Unlock()
+    // and purely rely on AddDirtyRect to notify D3D9 that contents have changed.
+    // We have no way of knowing which mip levels were actually changed.
+    m_texture.SetAllNeedUpload();
     return D3D_OK;
   }
 
@@ -158,7 +163,12 @@ namespace dxvk {
   }
 
   HRESULT STDMETHODCALLTYPE D3D9Texture3D::AddDirtyBox(CONST D3DBOX* pDirtyBox) {
-    m_texture.AddUpdateDirtyBox(pDirtyBox, 0);
+    m_texture.AddDirtyBox(pDirtyBox, 0);
+
+    // Some games keep using the pointer returned in LockBox() after calling Unlock()
+    // and purely rely on AddDirtyBox to notify D3D9 that contents have changed.
+    // We have no way of knowing which mip levels were actually changed.
+    m_texture.SetAllNeedUpload();
     return D3D_OK;
   }
 
@@ -238,9 +248,16 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE D3D9TextureCube::AddDirtyRect(D3DCUBEMAP_FACES Face, CONST RECT* pDirtyRect) {
     if (pDirtyRect) {
       D3DBOX box = { UINT(pDirtyRect->left), UINT(pDirtyRect->top), UINT(pDirtyRect->right), UINT(pDirtyRect->bottom), 0, 1 };
-      m_texture.AddUpdateDirtyBox(&box, Face);
+      m_texture.AddDirtyBox(&box, Face);
     } else {
-      m_texture.AddUpdateDirtyBox(nullptr, Face);
+      m_texture.AddDirtyBox(nullptr, Face);
+    }
+
+    // Some games keep using the pointer returned in LockRect() after calling Unlock()
+    // and purely rely on AddDirtyRect to notify D3D9 that contents have changed.
+    // We have no way of knowing which mip levels were actually changed.
+    for (uint32_t m = 0; m < m_texture.Desc()->MipLevels; m++) {
+      m_texture.SetNeedsUpload(m_texture.CalcSubresource(Face, m), true);
     }
     return D3D_OK;
   }
