@@ -302,13 +302,13 @@ namespace dxvk {
      && options->maxSharedMemory < sharedMemory)
       sharedMemory = options->maxSharedMemory;
     
-    #ifndef _WIN64
-    // The value returned by DXGI is a 32-bit value
-    // on 32-bit platforms, so we need to clamp it
-    VkDeviceSize maxMemory = 0xC0000000;
-    deviceMemory = std::min(deviceMemory, maxMemory);
-    sharedMemory = std::min(sharedMemory, maxMemory);
-    #endif
+    if (env::is32BitHostPlatform()) {
+      // The value returned by DXGI is a 32-bit value
+      // on 32-bit platforms, so we need to clamp it
+      VkDeviceSize maxMemory = 0xC0000000;
+      deviceMemory = std::min(deviceMemory, maxMemory);
+      sharedMemory = std::min(sharedMemory, maxMemory);
+    }
     
     pDesc->VendorId                       = deviceProp.vendorID;
     pDesc->DeviceId                       = deviceProp.deviceID;
