@@ -265,7 +265,8 @@ namespace dxvk {
     { VK_FORMAT_R16_SFLOAT,
       VK_FORMAT_D16_UNORM,
       VK_FORMAT_R16_UINT,
-      VK_IMAGE_ASPECT_COLOR_BIT },
+      VK_IMAGE_ASPECT_COLOR_BIT,
+      VK_IMAGE_ASPECT_DEPTH_BIT },
     // DXGI_FORMAT_D16_UNORM
     { VK_FORMAT_UNDEFINED,
       VK_FORMAT_D16_UNORM,
@@ -853,7 +854,7 @@ namespace dxvk {
     if (!CheckImageFormatSupport(adapter, VK_FORMAT_D24_UNORM_S8_UINT,
           VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT |
           VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT)) {
-      Logger::warn("DXGI: VK_FORMAT_D24_UNORM_S8_UINT -> VK_FORMAT_D32_SFLOAT_S8_UINT");
+      Logger::info("DXGI: VK_FORMAT_D24_UNORM_S8_UINT -> VK_FORMAT_D32_SFLOAT_S8_UINT");
       RemapDepthFormat(DXGI_FORMAT_R24G8_TYPELESS,        VK_FORMAT_D32_SFLOAT_S8_UINT);
       RemapDepthFormat(DXGI_FORMAT_R24_UNORM_X8_TYPELESS, VK_FORMAT_D32_SFLOAT_S8_UINT);
       RemapDepthFormat(DXGI_FORMAT_X24_TYPELESS_G8_UINT,  VK_FORMAT_D32_SFLOAT_S8_UINT);
