@@ -4,6 +4,7 @@
 #include "dxso_header.h"
 #include "dxso_modinfo.h"
 #include "dxso_isgn.h"
+#include "dxso_util.h"
 
 #include "../d3d9/d3d9_constant_layout.h"
 #include "../d3d9/d3d9_shader_permutations.h"
@@ -128,8 +129,7 @@ namespace dxvk {
     DxsoSamplerInfo color[SamplerTypeCount];
     DxsoSamplerInfo depth[SamplerTypeCount];
 
-    uint32_t colorSpecConst;
-    uint32_t depthSpecConst;
+    uint32_t boundConst;
 
     DxsoTextureType type;
   };
@@ -256,6 +256,7 @@ namespace dxvk {
     const DxsoDefinedConstants& constants() { return m_constants; }
     uint32_t usedSamplers() const { return m_usedSamplers; }
     uint32_t usedRTs() const { return m_usedRTs; }
+    uint32_t maxDefinedConstant() const { return m_maxDefinedConstant; }
 
   private:
 
@@ -266,10 +267,12 @@ namespace dxvk {
 
     DxsoShaderMetaInfo         m_meta;
     DxsoDefinedConstants       m_constants;
+    uint32_t                   m_maxDefinedConstant;
 
     SpirvModule                m_module;
 
     uint32_t                   m_boolSpecConstant;
+    uint32_t                   m_depthSpecConstant;
 
     ///////////////////////////////////////////////////////
     // Resource slot description for the shader. This will
@@ -302,7 +305,10 @@ namespace dxvk {
 
     ////////////////////////////////////
     // Ptr to the constant buffer array
-    uint32_t m_cBuffer;
+    uint32_t m_cBuffer = 0;
+    uint32_t m_cFloatBuffer = 0;
+    uint32_t m_cIntBuffer = 0;
+    uint32_t m_cBoolBuffer = 0;
 
     ////////////////////////////////////////
     // Constant buffer deffed mappings
@@ -373,6 +379,9 @@ namespace dxvk {
 
     //////////////////////
     // Common shader dcls
+    template<DxsoConstantBufferType ConstantBufferType>
+    int emitDclSwvpConstantBuffer();
+
     void emitDclConstantBuffer();
 
     void emitDclInputArray();
@@ -536,10 +545,27 @@ namespace dxvk {
     DxsoRegisterValue emitSaturate(
             DxsoRegisterValue       srcValue);
 
+    DxsoRegisterValue emitMulOperand(
+            DxsoRegisterValue       operand,
+            DxsoRegisterValue       other);
+
+    DxsoRegisterValue emitMul(
+            DxsoRegisterValue       a,
+            DxsoRegisterValue       b);
+
+    DxsoRegisterValue emitFma(
+            DxsoRegisterValue       a,
+            DxsoRegisterValue       b,
+            DxsoRegisterValue       c);
+
     DxsoRegisterValue emitDot(
             DxsoRegisterValue       a,
             DxsoRegisterValue       b);
 
+    DxsoRegisterValue emitCross(
+            DxsoRegisterValue       a,
+            DxsoRegisterValue       b);
+
     DxsoRegisterValue emitRegisterInsert(
             DxsoRegisterValue       dstValue,
             DxsoRegisterValue       srcValue,
@@ -638,6 +664,7 @@ namespace dxvk {
     void emitTexCoord(const DxsoInstructionContext& ctx);
     void emitTextureSample(const DxsoInstructionContext& ctx);
     void emitTextureKill(const DxsoInstructionContext& ctx);
+    void emitTextureDepth(const DxsoInstructionContext& ctx);
 
     uint32_t emitSample(
             bool                    projected,
@@ -677,6 +704,11 @@ namespace dxvk {
     uint32_t getPointerTypeId(
       const DxsoRegisterInfo& type);
 
+
+    bool isSwvp() {
+      return m_layout->bitmaskCount != 1;
+    }
+
   };
 
 }
\ No newline at end of file
