@@ -22,17 +22,11 @@ namespace dxvk {
       && (devInfo.coreSubgroup.supportedStages & VK_SHADER_STAGE_FRAGMENT_BIT)
       && (devInfo.coreSubgroup.supportedOperations & VK_SUBGROUP_FEATURE_BALLOT_BIT);
 
-    // Disable early discard on RADV (with LLVM) due to GPU hangs
     // Disable early discard on Nvidia because it may hurt performance
-    bool isRadvAco = std::string(devInfo.core.properties.deviceName).find("RADV/ACO") != std::string::npos;
-
-    if ((adapter->matchesDriver(DxvkGpuVendor::Amd,    VK_DRIVER_ID_MESA_RADV_KHR,          0, 0) && !isRadvAco)
-     || (adapter->matchesDriver(DxvkGpuVendor::Nvidia, VK_DRIVER_ID_NVIDIA_PROPRIETARY_KHR, 0, 0)))
+    if (adapter->matchesDriver(DxvkGpuVendor::Nvidia, VK_DRIVER_ID_NVIDIA_PROPRIETARY_KHR, 0, 0))
       useSubgroupOpsForEarlyDiscard = false;
     
     // Apply shader-related options
-    applyTristate(useSubgroupOpsForEarlyDiscard, device->config().useEarlyDiscard);
-
     strictConstantCopies = options.strictConstantCopies;
 
     strictPow            = options.strictPow;
@@ -44,11 +38,13 @@ namespace dxvk {
 
     forceSamplerTypeSpecConstants = options.forceSamplerTypeSpecConstants;
 
-    vertexConstantBufferAsSSBO = pDevice->GetVertexConstantLayout().totalSize() > devInfo.core.properties.limits.maxUniformBufferRange;
+    vertexFloatConstantBufferAsSSBO = pDevice->GetVertexConstantLayout().floatSize() > devInfo.core.properties.limits.maxUniformBufferRange;
 
     longMad = options.longMad;
 
     alphaTestWiggleRoom = options.alphaTestWiggleRoom;
+
+    robustness2Supported = devFeatures.extRobustness2.robustBufferAccess2;
   }
 
 }
\ No newline at end of file
