@@ -197,6 +197,12 @@ namespace dxvk {
      */
     bool isUnifiedMemoryArchitecture() const;
 
+    /**
+     * \brief Queries default framebuffer size
+     * \returns Default framebuffer size
+     */
+    DxvkFramebufferSize getDefaultFramebufferSize() const;
+
     /**
      * \brief Queries supported shader stages
      * \returns Supported shader pipeline stages
@@ -264,13 +270,11 @@ namespace dxvk {
     /**
      * \brief Creates framebuffer for a set of render targets
      * 
-     * Automatically deduces framebuffer dimensions
-     * from the supplied render target views.
-     * \param [in] renderTargets Render targets
+     * \param [in] info Framebuffer info
      * \returns The framebuffer object
      */
     Rc<DxvkFramebuffer> createFramebuffer(
-      const DxvkRenderTargets& renderTargets);
+      const DxvkFramebufferInfo&  info);
     
     /**
      * \brief Creates a buffer object
@@ -399,12 +403,10 @@ namespace dxvk {
      * the submission thread. The status of this operation
      * can be retrieved with \ref waitForSubmission.
      * \param [in] presenter The presenter
-     * \param [in] semaphore Sync semaphore
      * \param [out] status Present status
      */
     void presentImage(
       const Rc<vk::Presenter>&        presenter,
-            VkSemaphore               semaphore,
             DxvkSubmitStatus*         status);
     
     /**
