@@ -20,12 +20,8 @@ namespace dxvk {
     m_options = DxvkOptions(m_config);
 
     m_extProviders.push_back(&DxvkPlatformExts::s_instance);
-
-    if (m_options.enableOpenVR)
-      m_extProviders.push_back(&VrInstance::s_instance);
-
-    if (m_options.enableOpenXR)
-      m_extProviders.push_back(&DxvkXrProvider::s_instance);
+    m_extProviders.push_back(&VrInstance::s_instance);
+    m_extProviders.push_back(&DxvkXrProvider::s_instance);
 
     Logger::info("Built-in extension providers:");
     for (const auto& provider : m_extProviders)
@@ -130,7 +126,7 @@ namespace dxvk {
     appInfo.pApplicationName      = appName.c_str();
     appInfo.applicationVersion    = 0;
     appInfo.pEngineName           = "DXVK";
-    appInfo.engineVersion         = VK_MAKE_VERSION(1, 9, 2);
+    appInfo.engineVersion         = VK_MAKE_VERSION(1, 9, 4);
     appInfo.apiVersion            = VK_MAKE_VERSION(1, 1, 0);
     
     VkInstanceCreateInfo info;
