@@ -1,3 +1,5 @@
+#include <algorithm>
+
 #include "dxvk_device.h"
 #include "dxvk_memory.h"
 
@@ -58,8 +60,9 @@ namespace dxvk {
   DxvkMemoryChunk::DxvkMemoryChunk(
           DxvkMemoryAllocator*  alloc,
           DxvkMemoryType*       type,
-          DxvkDeviceMemory      memory)
-  : m_alloc(alloc), m_type(type), m_memory(memory) {
+          DxvkDeviceMemory      memory,
+          DxvkMemoryFlags       hints)
+  : m_alloc(alloc), m_type(type), m_memory(memory), m_hints(hints) {
     // Mark the entire chunk as free
     m_freeList.push_back(FreeSlice { 0, memory.memSize });
   }
@@ -76,11 +79,10 @@ namespace dxvk {
           VkMemoryPropertyFlags flags,
           VkDeviceSize          size,
           VkDeviceSize          align,
-          float                 priority) {
+          DxvkMemoryFlags       hints) {
     // Property flags must be compatible. This could
     // be refined a bit in the future if necessary.
-    if (m_memory.memFlags != flags
-     || m_memory.priority != priority)
+    if (m_memory.memFlags != flags || !checkHints(hints))
       return DxvkMemory();
     
     // If the chunk is full, return
@@ -152,6 +154,30 @@ namespace dxvk {
   }
   
   
+  bool DxvkMemoryChunk::isEmpty() const {
+    return m_freeList.size() == 1
+        && m_freeList[0].length == m_memory.memSize;
+  }
+
+
+  bool DxvkMemoryChunk::isCompatible(const Rc<DxvkMemoryChunk>& other) const {
+    return other->m_memory.memFlags == m_memory.memFlags && other->m_hints == m_hints;
+  }
+
+
+  bool DxvkMemoryChunk::checkHints(DxvkMemoryFlags hints) const {
+    DxvkMemoryFlags mask(
+      DxvkMemoryFlag::Small,
+      DxvkMemoryFlag::GpuReadable,
+      DxvkMemoryFlag::GpuWritable);
+
+    if (hints.test(DxvkMemoryFlag::IgnoreConstraints))
+      mask = DxvkMemoryFlags();
+
+    return (m_hints & mask) == (hints & mask);
+  }
+
+
   DxvkMemoryAllocator::DxvkMemoryAllocator(const DxvkDevice* device)
   : m_vkd             (device->vkd()),
     m_device          (device),
@@ -174,31 +200,35 @@ namespace dxvk {
       m_memTypes[i].heapId     = m_memProps.memoryTypes[i].heapIndex;
       m_memTypes[i].memType    = m_memProps.memoryTypes[i];
       m_memTypes[i].memTypeId  = i;
-      m_memTypes[i].chunkSize  = pickChunkSize(i);
+    }
+
+    /* Check what kind of heap the HVV memory type is on, if any. If the
+     * HVV memory type is on the largest device-local heap, we either have
+     * an UMA system or an RBAR-enabled system. Otherwise, there will likely
+     * be a separate, smaller heap for it. */
+    VkDeviceSize largestDeviceLocalHeap = 0;
+
+    for (uint32_t i = 0; i < m_memProps.memoryTypeCount; i++) {
+      if (m_memTypes[i].memType.propertyFlags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)
+        largestDeviceLocalHeap = std::max(largestDeviceLocalHeap, m_memTypes[i].heap->properties.size);
     }
 
     /* Work around an issue on Nvidia drivers where using the entire
-     * device_local | host_visible heap can cause crashes, presumably
-     * due to subsequent internal driver allocations failing */
-    bool nvidiaBug3114283Active = false;
+     * device_local | host_visible heap can cause crashes or slowdowns */
+    if (m_device->properties().core.properties.vendorID == uint16_t(DxvkGpuVendor::Nvidia)) {
+      bool shrinkNvidiaHvvHeap = device->adapter()->matchesDriver(DxvkGpuVendor::Nvidia,
+        VK_DRIVER_ID_NVIDIA_PROPRIETARY_KHR, 0, VK_MAKE_VERSION(465, 0, 0));
 
-    // Fix is available in mainline drivers starting with the 465 driver series.
-    if (device->adapter()->matchesDriver(DxvkGpuVendor::Nvidia,
-                                         VK_DRIVER_ID_NVIDIA_PROPRIETARY_KHR,
-                                         0,
-                                         VK_MAKE_VERSION(465, 0, 0))) {
-      nvidiaBug3114283Active = true;
-    }
+      applyTristate(shrinkNvidiaHvvHeap, device->config().shrinkNvidiaHvvHeap);
 
-    applyTristate(nvidiaBug3114283Active, device->config().halveNvidiaHVVHeap);
+      if (shrinkNvidiaHvvHeap) {
+        for (uint32_t i = 0; i < m_memProps.memoryTypeCount; i++) {
+          VkMemoryPropertyFlags hvvFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT | VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT;
 
-    if ((m_device->properties().core.properties.vendorID == uint16_t(DxvkGpuVendor::Nvidia))
-     && (nvidiaBug3114283Active)) {
-      for (uint32_t i = 0; i < m_memProps.memoryTypeCount; i++) {
-        constexpr VkMemoryPropertyFlags flags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT | VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT;
-
-        if ((m_memTypes[i].memType.propertyFlags & flags) == flags)
-          m_memTypes[i].heap->budget = m_memTypes[i].heap->properties.size / 2;
+          if ((m_memTypes[i].memType.propertyFlags & hvvFlags) == hvvFlags
+           && (m_memTypes[i].heap->properties.size < largestDeviceLocalHeap))
+            m_memTypes[i].heap->budget = 32 << 20;
+        }
       }
     }
   }
@@ -214,19 +244,38 @@ namespace dxvk {
     const VkMemoryDedicatedRequirements&    dedAllocReq,
     const VkMemoryDedicatedAllocateInfo&    dedAllocInfo,
           VkMemoryPropertyFlags             flags,
-          float                             priority) {
+          DxvkMemoryFlags                   hints) {
     std::lock_guard<dxvk::mutex> lock(m_mutex);
 
+    // Keep small allocations together to avoid fragmenting
+    // chunks for larger resources with lots of small gaps,
+    // as well as resources with potentially weird lifetimes
+    if (req->size <= SmallAllocationThreshold) {
+      hints.set(DxvkMemoryFlag::Small);
+      hints.clr(DxvkMemoryFlag::GpuWritable, DxvkMemoryFlag::GpuReadable);
+    }
+
+    // Ignore all hints for host-visible allocations since they
+    // usually don't make much sense for those resources
+    if (flags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
+      hints = DxvkMemoryFlags();
+
     // Try to allocate from a memory type which supports the given flags exactly
     auto dedAllocPtr = dedAllocReq.prefersDedicatedAllocation ? &dedAllocInfo : nullptr;
-    DxvkMemory result = this->tryAlloc(req, dedAllocPtr, flags, priority);
+    DxvkMemory result = this->tryAlloc(req, dedAllocPtr, flags, hints);
 
     // If the first attempt failed, try ignoring the dedicated allocation
     if (!result && dedAllocPtr && !dedAllocReq.requiresDedicatedAllocation) {
-      result = this->tryAlloc(req, nullptr, flags, priority);
+      result = this->tryAlloc(req, nullptr, flags, hints);
       dedAllocPtr = nullptr;
     }
 
+    // Retry without the hint constraints
+    if (!result) {
+      hints.set(DxvkMemoryFlag::IgnoreConstraints);
+      result = this->tryAlloc(req, nullptr, flags, hints);
+    }
+
     // If that still didn't work, probe slower memory types as well
     VkMemoryPropertyFlags optFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT
                                    | VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
@@ -236,7 +285,7 @@ namespace dxvk {
       remFlags |= optFlags & -optFlags;
       optFlags &= ~remFlags;
 
-      result = this->tryAlloc(req, dedAllocPtr, flags & ~remFlags, priority);
+      result = this->tryAlloc(req, dedAllocPtr, flags & ~remFlags, hints);
     }
     
     if (!result) {
@@ -273,7 +322,7 @@ namespace dxvk {
     const VkMemoryRequirements*             req,
     const VkMemoryDedicatedAllocateInfo*    dedAllocInfo,
           VkMemoryPropertyFlags             flags,
-          float                             priority) {
+          DxvkMemoryFlags                   hints) {
     DxvkMemory result;
 
     for (uint32_t i = 0; i < m_memProps.memoryTypeCount && !result; i++) {
@@ -282,7 +331,7 @@ namespace dxvk {
       
       if (supported && adequate) {
         result = this->tryAllocFromType(&m_memTypes[i],
-          flags, req->size, req->alignment, priority, dedAllocInfo);
+          flags, req->size, req->alignment, hints, dedAllocInfo);
       }
     }
     
@@ -295,35 +344,37 @@ namespace dxvk {
           VkMemoryPropertyFlags             flags,
           VkDeviceSize                      size,
           VkDeviceSize                      align,
-          float                             priority,
+          DxvkMemoryFlags                   hints,
     const VkMemoryDedicatedAllocateInfo*    dedAllocInfo) {
-    // Prevent unnecessary external host memory fragmentation
-    bool isDeviceLocal = (flags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) != 0;
-
-    if (!isDeviceLocal)
-      priority = 0.0f;
+    VkDeviceSize chunkSize = pickChunkSize(type->memTypeId, hints);
 
     DxvkMemory memory;
 
-    if (size >= type->chunkSize || dedAllocInfo) {
+    if (size >= chunkSize || dedAllocInfo) {
+      if (this->shouldFreeEmptyChunks(type->heap, size))
+        this->freeEmptyChunks(type->heap);
+
       DxvkDeviceMemory devMem = this->tryAllocDeviceMemory(
-        type, flags, size, priority, dedAllocInfo);
+        type, flags, size, hints, dedAllocInfo);
 
       if (devMem.memHandle != VK_NULL_HANDLE)
         memory = DxvkMemory(this, nullptr, type, devMem.memHandle, 0, size, devMem.memPointer);
     } else {
       for (uint32_t i = 0; i < type->chunks.size() && !memory; i++)
-        memory = type->chunks[i]->alloc(flags, size, align, priority);
+        memory = type->chunks[i]->alloc(flags, size, align, hints);
       
       if (!memory) {
         DxvkDeviceMemory devMem;
         
-        for (uint32_t i = 0; i < 6 && (type->chunkSize >> i) >= size && !devMem.memHandle; i++)
-          devMem = tryAllocDeviceMemory(type, flags, type->chunkSize >> i, priority, nullptr);
+        if (this->shouldFreeEmptyChunks(type->heap, chunkSize))
+          this->freeEmptyChunks(type->heap);
+
+        for (uint32_t i = 0; i < 6 && (chunkSize >> i) >= size && !devMem.memHandle; i++)
+          devMem = tryAllocDeviceMemory(type, flags, chunkSize >> i, hints, nullptr);
 
         if (devMem.memHandle) {
-          Rc<DxvkMemoryChunk> chunk = new DxvkMemoryChunk(this, type, devMem);
-          memory = chunk->alloc(flags, size, align, priority);
+          Rc<DxvkMemoryChunk> chunk = new DxvkMemoryChunk(this, type, devMem, hints);
+          memory = chunk->alloc(flags, size, align, hints);
 
           type->chunks.push_back(std::move(chunk));
         }
@@ -341,7 +392,7 @@ namespace dxvk {
           DxvkMemoryType*                   type,
           VkMemoryPropertyFlags             flags,
           VkDeviceSize                      size,
-          float                             priority,
+          DxvkMemoryFlags                   hints,
     const VkMemoryDedicatedAllocateInfo*    dedAllocInfo) {
     bool useMemoryPriority = (flags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)
                           && (m_device->features().extMemoryPriority.memoryPriority);
@@ -349,6 +400,13 @@ namespace dxvk {
     if (type->heap->budget && type->heap->stats.memoryAllocated + size > type->heap->budget)
       return DxvkDeviceMemory();
 
+    float priority = 0.0f;
+
+    if (hints.test(DxvkMemoryFlag::GpuReadable))
+      priority = 0.5f;
+    if (hints.test(DxvkMemoryFlag::GpuWritable))
+      priority = 1.0f;
+
     DxvkDeviceMemory result;
     result.memSize  = size;
     result.memFlags = flags;
@@ -411,6 +469,18 @@ namespace dxvk {
           VkDeviceSize          offset,
           VkDeviceSize          length) {
     chunk->free(offset, length);
+
+    if (chunk->isEmpty()) {
+      Rc<DxvkMemoryChunk> chunkRef = chunk;
+
+      // Free the chunk if we have to, or at least put it at the end of
+      // the list so that chunks that are already in use and cannot be
+      // freed are prioritized for allocations to reduce memory pressure.
+      type->chunks.erase(std::remove(type->chunks.begin(), type->chunks.end(), chunkRef));
+
+      if (!this->shouldFreeChunk(type, chunkRef))
+        type->chunks.push_back(std::move(chunkRef));
+    }
   }
   
 
@@ -423,19 +493,20 @@ namespace dxvk {
   }
 
 
-  VkDeviceSize DxvkMemoryAllocator::pickChunkSize(uint32_t memTypeId) const {
+  VkDeviceSize DxvkMemoryAllocator::pickChunkSize(uint32_t memTypeId, DxvkMemoryFlags hints) const {
     VkMemoryType type = m_memProps.memoryTypes[memTypeId];
     VkMemoryHeap heap = m_memProps.memoryHeaps[type.heapIndex];
 
     // Default to a chunk size of 128 MiB
     VkDeviceSize chunkSize = 128 << 20;
 
-    // Try to waste a bit less system memory in 32-bit
-    // applications due to address space constraints
-    #ifndef _WIN64
+    if (hints.test(DxvkMemoryFlag::Small))
+      chunkSize = 16 << 20;
+
+    // Try to waste a bit less system memory especially in
+    // 32-bit applications due to address space constraints
     if (type.propertyFlags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
-      chunkSize = 32 << 20;
-    #endif
+      chunkSize = 16 << 20;
 
     // Reduce the chunk size on small heaps so
     // we can at least fit in 15 allocations
@@ -444,5 +515,51 @@ namespace dxvk {
 
     return chunkSize;
   }
-  
+
+
+  bool DxvkMemoryAllocator::shouldFreeChunk(
+    const DxvkMemoryType*       type,
+    const Rc<DxvkMemoryChunk>&  chunk) const {
+    // Under memory pressure, we should start freeing everything.
+    if (this->shouldFreeEmptyChunks(type->heap, 0))
+      return true;
+
+    // Even if we have enough memory to spare, only keep
+    // one chunk of each type around to save memory.
+    for (const auto& c : type->chunks) {
+      if (c != chunk && c->isEmpty() && c->isCompatible(chunk))
+        return true;
+    }
+
+    return false;
+  }
+
+
+  bool DxvkMemoryAllocator::shouldFreeEmptyChunks(
+    const DxvkMemoryHeap*       heap,
+          VkDeviceSize          allocationSize) const {
+    VkDeviceSize budget = heap->budget;
+
+    if (!budget)
+      budget = (heap->properties.size * 4) / 5;
+
+    return heap->stats.memoryAllocated + allocationSize > budget;
+  }
+
+
+  void DxvkMemoryAllocator::freeEmptyChunks(
+    const DxvkMemoryHeap*       heap) {
+    for (uint32_t i = 0; i < m_memProps.memoryTypeCount; i++) {
+      DxvkMemoryType* type = &m_memTypes[i];
+
+      if (type->heap != heap)
+        continue;
+
+      type->chunks.erase(
+        std::remove_if(type->chunks.begin(), type->chunks.end(),
+          [] (const Rc<DxvkMemoryChunk>& chunk) { return chunk->isEmpty(); }),
+        type->chunks.end());
+    }
+  }
+
 }
