@@ -62,8 +62,6 @@ namespace dxvk {
     VkMemoryType      memType;
     uint32_t          memTypeId;
 
-    VkDeviceSize      chunkSize;
-
     std::vector<Rc<DxvkMemoryChunk>> chunks;
   };
   
@@ -155,6 +153,22 @@ namespace dxvk {
     void free();
     
   };
+
+
+  /**
+   * \brief Memory allocation flags
+   *
+   * Used to batch similar allocations into the same
+   * set of chunks, which may help with fragmentation.
+   */
+  enum class DxvkMemoryFlag : uint32_t {
+    Small             = 0,  ///< Small allocation
+    GpuReadable       = 1,  ///< Medium-priority resource
+    GpuWritable       = 2,  ///< High-priority resource
+    IgnoreConstraints = 3,  ///< Ignore most allocation flags
+  };
+
+  using DxvkMemoryFlags = Flags<DxvkMemoryFlag>;
   
   
   /**
@@ -170,7 +184,8 @@ namespace dxvk {
     DxvkMemoryChunk(
             DxvkMemoryAllocator*  alloc,
             DxvkMemoryType*       type,
-            DxvkDeviceMemory      memory);
+            DxvkDeviceMemory      memory,
+            DxvkMemoryFlags       m_hints);
     
     ~DxvkMemoryChunk();
 
@@ -179,17 +194,17 @@ namespace dxvk {
      * 
      * On failure, this returns a slice with
      * \c VK_NULL_HANDLE as the memory handle.
-     * \param [in] flags Requested memory flags
+     * \param [in] flags Requested memory type flags
      * \param [in] size Number of bytes to allocate
      * \param [in] align Required alignment
-     * \param [in] priority Requested priority
+     * \param [in] hints Memory category
      * \returns The allocated memory slice
      */
     DxvkMemory alloc(
             VkMemoryPropertyFlags flags,
             VkDeviceSize          size,
             VkDeviceSize          align,
-            float                 priority);
+            DxvkMemoryFlags       hints);
     
     /**
      * \brief Frees memory
@@ -203,7 +218,19 @@ namespace dxvk {
     void free(
             VkDeviceSize  offset,
             VkDeviceSize  length);
-    
+
+    /**
+     * \brief Checks whether the chunk is being used
+     * \returns \c true if there are no allocations left
+     */
+    bool isEmpty() const;
+
+    /**
+     * \brief Checks whether hints and flags of another chunk match
+     * \param [in] other The chunk to compare to
+     */
+    bool isCompatible(const Rc<DxvkMemoryChunk>& other) const;
+
   private:
     
     struct FreeSlice {
@@ -214,8 +241,11 @@ namespace dxvk {
     DxvkMemoryAllocator*  m_alloc;
     DxvkMemoryType*       m_type;
     DxvkDeviceMemory      m_memory;
+    DxvkMemoryFlags       m_hints;
     
     std::vector<FreeSlice> m_freeList;
+
+    bool checkHints(DxvkMemoryFlags hints) const;
     
   };
   
@@ -229,6 +259,8 @@ namespace dxvk {
   class DxvkMemoryAllocator {
     friend class DxvkMemory;
     friend class DxvkMemoryChunk;
+
+    constexpr static VkDeviceSize SmallAllocationThreshold = 256 << 10;
   public:
     
     DxvkMemoryAllocator(const DxvkDevice* device);
@@ -253,7 +285,7 @@ namespace dxvk {
      * \param [in] dedAllocReq Dedicated allocation requirements
      * \param [in] dedAllocInfo Dedicated allocation info
      * \param [in] flags Memory type flags
-     * \param [in] priority Device-local memory priority
+     * \param [in] hints Memory hints
      * \returns Allocated memory slice
      */
     DxvkMemory alloc(
@@ -261,7 +293,7 @@ namespace dxvk {
       const VkMemoryDedicatedRequirements&    dedAllocReq,
       const VkMemoryDedicatedAllocateInfo&    dedAllocInfo,
             VkMemoryPropertyFlags             flags,
-            float                             priority);
+            DxvkMemoryFlags                   hints);
     
     /**
      * \brief Queries memory stats
@@ -282,7 +314,7 @@ namespace dxvk {
     const VkPhysicalDeviceProperties       m_devProps;
     const VkPhysicalDeviceMemoryProperties m_memProps;
     
-    std::mutex                                      m_mutex;
+    dxvk::mutex                                     m_mutex;
     std::array<DxvkMemoryHeap, VK_MAX_MEMORY_HEAPS> m_memHeaps;
     std::array<DxvkMemoryType, VK_MAX_MEMORY_TYPES> m_memTypes;
 
@@ -290,21 +322,21 @@ namespace dxvk {
       const VkMemoryRequirements*             req,
       const VkMemoryDedicatedAllocateInfo*    dedAllocInfo,
             VkMemoryPropertyFlags             flags,
-            float                             priority);
+            DxvkMemoryFlags                   hints);
     
     DxvkMemory tryAllocFromType(
             DxvkMemoryType*                   type,
             VkMemoryPropertyFlags             flags,
             VkDeviceSize                      size,
             VkDeviceSize                      align,
-            float                             priority,
+            DxvkMemoryFlags                   hints,
       const VkMemoryDedicatedAllocateInfo*    dedAllocInfo);
     
     DxvkDeviceMemory tryAllocDeviceMemory(
             DxvkMemoryType*                   type,
             VkMemoryPropertyFlags             flags,
             VkDeviceSize                      size,
-            float                             priority,
+            DxvkMemoryFlags                   hints,
       const VkMemoryDedicatedAllocateInfo*    dedAllocInfo);
     
     void free(
@@ -321,7 +353,19 @@ namespace dxvk {
             DxvkDeviceMemory      memory);
     
     VkDeviceSize pickChunkSize(
-            uint32_t              memTypeId) const;
+            uint32_t              memTypeId,
+            DxvkMemoryFlags       hints) const;
+
+    bool shouldFreeChunk(
+      const DxvkMemoryType*       type,
+      const Rc<DxvkMemoryChunk>&  chunk) const;
+
+    bool shouldFreeEmptyChunks(
+      const DxvkMemoryHeap*       heap,
+            VkDeviceSize          allocationSize) const;
+
+    void freeEmptyChunks(
+      const DxvkMemoryHeap*       heap);
 
   };
   
