@@ -11,22 +11,18 @@ namespace dxvk {
     /// Enable state cache
     bool enableStateCache;
 
-    /// Enables OpenVR loading
-    bool enableOpenVR;
-
-    /// Enables OpenXR loading
-    bool enableOpenXR;
-
     /// Number of compiler threads
     /// when using the state cache
     int32_t numCompilerThreads;
 
     /// Shader-related options
     Tristate useRawSsbo;
-    Tristate useEarlyDiscard;
+
+    /// Workaround for NVIDIA driver bug 3114283
+    Tristate shrinkNvidiaHvvHeap;
 
     /// HUD elements
     std::string hud;
   };
 
-}
\ No newline at end of file
+}
