@@ -93,7 +93,8 @@ namespace dxvk::hud {
     
     void beginFrame(
       const Rc<DxvkContext>&  context,
-            VkExtent2D        surfaceSize);
+            VkExtent2D        surfaceSize,
+            float             scale);
     
     void drawText(
             float             size,
@@ -108,6 +109,10 @@ namespace dxvk::hud {
     VkExtent2D surfaceSize() const {
       return m_surfaceSize;
     }
+
+    float scale() const {
+      return m_scale;
+    }
     
   private:
     
@@ -125,6 +130,7 @@ namespace dxvk::hud {
     std::array<uint8_t, 256> m_charMap;
     
     Mode                m_mode;
+    float               m_scale;
     VkExtent2D          m_surfaceSize;
     Rc<DxvkContext>     m_context;
     
