@@ -139,14 +139,6 @@ namespace dxvk {
     { R"(\\SaintsRowTheThird_DX11\.exe$)", {{
       { "d3d11.constantBufferRangeCheck",   "True" },
     }} },
-    /* Metal Gear Solid 5                         */
-    { R"(\\mgsvtpp\.exe$)", {{
-      { "dxvk.enableOpenVR",                "False" },
-    }} },
-    /* Raft                                       */
-    { R"(\\Raft\.exe$)", {{
-      { "dxvk.enableOpenVR",                "False" },
-    }} },
     /* Crysis 3 - slower if it notices AMD card     *
      * Apitrace mode helps massively in cpu bound   *
      * game parts                                   */
@@ -154,6 +146,12 @@ namespace dxvk {
       { "dxgi.customVendorId",              "10de" },
       { "d3d11.apitraceMode",               "True" },
     }} },
+    /* Crysis 3 Remastered                          *
+     * Apitrace mode helps massively in cpu bound   *
+     * game parts                                   */
+    { R"(\\Crysis3Remastered\.exe$)", {{
+      { "d3d11.apitraceMode",               "True" },
+    }} },
     /* Atelier series - games try to render video *
      * with a D3D9 swap chain over the DXGI swap  *
      * chain, which breaks D3D11 presentation     */
@@ -189,10 +187,6 @@ namespace dxvk {
     { R"(\\F1_20(1[89]|[2-9][0-9])\.exe$)", {{
       { "d3d11.forceTgsmBarriers",          "True" },
     }} },
-    /* Subnautica                                 */
-    { R"(\\Subnautica\.exe$)", {{
-      { "dxvk.enableOpenVR",                "False" },
-    }} },
     /* Blue Reflection                            */
     { R"(\\BLUE_REFLECTION\.exe$)", {{
       { "d3d11.constantBufferRangeCheck",   "True" },
@@ -247,6 +241,17 @@ namespace dxvk {
     { R"(\\WOFF\.exe$)", {{
       { "d3d11.disableMsaa",                "True" },
     }} },
+    /* Final Fantasy XIV - Stuttering on NV       */
+    { R"(\\ffxiv_dx11\.exe$)", {{
+      { "dxvk.shrinkNvidiaHvvHeap",         "True" },
+    }} },
+    /* God of War - relies on NVAPI/AMDAGS for    *
+     * barrier stuff, needs nvapi for DLSS        */
+    { R"(\\GoW\.exe$)", {{
+      { "d3d11.ignoreGraphicsBarriers",     "True" },
+      { "d3d11.relaxedBarriers",            "True" },
+      { "dxgi.nvapiHack",                   "False" },
+    }} },
 
     /**********************************************/
     /* D3D9 GAMES                                 */
@@ -295,12 +300,6 @@ namespace dxvk {
       { "d3d9.supportX4R4G4B4",             "False" },
       { "d3d9.maxAvailableMemory",          "2048" },
       { "d3d9.memoryTrackTest",             "True" },
-       // The Sims 2 will try to upload 1024 constants
-       // every frame otherwise, which it never uses
-       // causing a massive discard + upload.
-      { "d3d9.swvpFloatCount",              "384" },
-      { "d3d9.swvpIntCount",                "16" },
-      { "d3d9.swvpBoolCount",               "16" },
     }} },
     /* Dead Space uses the a NULL render target instead
        of a 1x1 one if DF24 is NOT supported      */
@@ -409,12 +408,6 @@ namespace dxvk {
       { "d3d9.deferSurfaceCreation",        "True" },
       { "d3d9.memoryTrackTest",             "True" },
     }} },
-    /* TrackMania Forever                        */
-    { R"(\\TmForever\.exe$)", {{
-      { "d3d9.swvpFloatCount",              "256" },
-      { "d3d9.swvpIntCount",                "16" },
-      { "d3d9.swvpBoolCount",               "16" },
-    }} },
     /* Mafia 2                                   */
     { R"(\\mafia2\.exe$)", {{
       { "d3d9.customVendorId",              "10de" },
@@ -461,6 +454,42 @@ namespace dxvk {
     { R"(\\FantasyGrounds\.exe$)", {{
       { "d3d9.noExplicitFrontBuffer",       "True" },
     }} },
+    /* Red Orchestra 2                           */
+    { R"(\\ROGame\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* Dark Souls II                            */
+    { R"(\\DarkSoulsII\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* Dogfight 1942                            */
+    { R"(\\Dogfight1942\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* Bayonetta                                */
+    { R"(\\Bayonetta\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* Rayman Origins                           */
+    { R"(\\Rayman Origins\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* Guilty Gear Xrd -Relevator-              */
+    { R"(\\GuiltyGearXrd\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* Richard Burns Rally                      */
+    { R"(\\RichardBurnsRally_SSE\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* BlazBlue Centralfiction                  */
+    { R"(\\BBCF\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* James Cameron's Avatar needs invariantPosition to fix black flickering vegetation */
+    { R"(\\Avatar\.exe$)", {{
+      { "d3d9.invariantPosition",              "True" },
+    }} },
   }};
 
 
@@ -637,8 +666,7 @@ namespace dxvk {
           I             begin,
           I             end,
           V&            value) {
-    std::transform(str.begin(), str.end(), str.begin(),
-      [] (unsigned char c) { return (c >= 'A' && c <= 'Z') ? (c + 'a' - 'A') : c; });
+    str = Config::toLower(str);
 
     for (auto i = begin; i != end; i++) {
       if (str == i->first) {
@@ -710,4 +738,10 @@ namespace dxvk {
     }
   }
 
+  std::string Config::toLower(std::string str) {
+    std::transform(str.begin(), str.end(), str.begin(),
+      [] (unsigned char c) { return (c >= 'A' && c <= 'Z') ? (c + 'a' - 'A') : c; });
+    return str;
+  }
+
 }
