@@ -102,6 +102,8 @@ namespace dxvk {
      */
     static Config getUserConfig();
 
+    static std::string toLower(std::string str);
+
   private:
 
     OptionMap m_options;
@@ -125,6 +127,13 @@ namespace dxvk {
       const std::string&  value,
             Tristate&     result);
 
+    template<typename I, typename V>
+    static bool parseStringOption(
+            std::string   str,
+            I             begin,
+            I             end,
+            V&            value);
+
   };
 
 
@@ -142,4 +151,4 @@ namespace dxvk {
     option |= state == Tristate::True;
   }
 
-}
\ No newline at end of file
+}
