@@ -1,3 +1,5 @@
+#include <numeric>
+
 #include "util_env.h"
 
 #include "./com/com_include.h"
@@ -13,8 +15,25 @@ namespace dxvk::env {
 
     return str::fromws(result.data());
   }
-  
-  
+
+
+  size_t matchFileExtension(const std::string& name, const char* ext) {
+    auto pos = name.find_last_of('.');
+
+    if (pos == std::string::npos)
+      return pos;
+
+    bool matches = std::accumulate(name.begin() + pos + 1, name.end(), true,
+      [&ext] (bool current, char a) {
+        if (a >= 'A' && a <= 'Z')
+          a += 'a' - 'A';
+        return current && *ext && a == *(ext++);
+      });
+
+    return matches ? pos : std::string::npos;
+  }
+
+
   std::string getExeName() {
     std::string fullPath = getExePath();
     auto n = fullPath.find_last_of('\\');
@@ -25,6 +44,17 @@ namespace dxvk::env {
   }
 
 
+  std::string getExeBaseName() {
+    auto exeName = getExeName();
+    auto extp = matchFileExtension(exeName, "exe");
+
+    if (extp != std::string::npos)
+      exeName.erase(extp);
+
+    return exeName;
+  }
+
+
   std::string getExePath() {
     std::vector<WCHAR> exePath;
     exePath.resize(MAX_PATH + 1);
